@@ -16,13 +16,14 @@
 # ******************************************************************************
 import tensorflow as tf
 
-from keras import layers
+from keras import layers, backend
+from keras.utils import conv_utils
 
-from .layers import deserialize_quant_object, Calibrable
-from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
+from .layers import deserialize_quant_object, Calibrable, CalibrableVariable
+from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH, floor_through, QFloat
 
 
-__all__ = ["PaddedConv2D", "QuantizedConv2D"]
+__all__ = ["PaddedConv2D", "QuantizedConv2D", "QuantizedConv2DTranspose"]
 
 
 def apply_padding(inputs, strides, kernel_size, padding_value):
@@ -68,6 +69,7 @@ class PaddedConv2D(layers.Conv2D):
         padding_value (float, optional): the value used when padding for the 'same' convolution
             type. If None, zero-padding is used. Defaults to None.
     """
+
     def __init__(self, *args, padding_value=None, **kwargs):
         super().__init__(*args, **kwargs)
         # Raise an error if the padding value is not a scalar
@@ -150,7 +152,6 @@ class QuantizedConv2D(Calibrable, layers.Conv2D):
         else:
             self._padding_value = None
 
-
     def call(self, inputs, training=None):
         # raise an error if the inputs are not FixedPoint or tf.Tensor
         if not isinstance(inputs, (FixedPoint, tf.Tensor)):
@@ -211,3 +212,116 @@ class QuantizedConv2D(Calibrable, layers.Conv2D):
         if self._padding_value is not None:
             config["padding_value"] = self._padding_value.to_float().numpy()
         return config
+
+
+@tf.keras.utils.register_keras_serializable()
+class QuantizedConv2DTranspose(Calibrable, layers.Conv2DTranspose):
+    """A transposed convolutional layer that operates on quantized inputs and weights.
+
+    Args:
+        quant_config (dict, optional): the serialized quantization configuration.
+            Defaults to empty configuration.
+    """
+
+    def __init__(self, *args, quant_config={}, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.quant_config = quant_config
+        self.out_quantizer = deserialize_quant_object(self.quant_config, "output_quantizer", False)
+        self.weight_quantizer = deserialize_quant_object(self.quant_config, "weight_quantizer",
+                                                         True)
+        if self.use_bias:
+            self.bias_quantizer = deserialize_quant_object(self.quant_config, "bias_quantizer",
+                                                           True)
+        self.buffer_bitwidth = self.quant_config.get("buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
+        assert self.buffer_bitwidth > 0, "The buffer_bitwidth must be a strictly positive integer."
+
+        # Add objects that will store the shift values.
+        self.input_shift = CalibrableVariable()
+        self.conv_shift = CalibrableVariable()
+        self.bias_shift = CalibrableVariable()
+
+    def call(self, inputs, training=None):
+        # Raise an error if the inputs are not FixedPoint
+        if not isinstance(inputs, FixedPoint):
+            raise TypeError("QuantizedConv2DTranspose only accepts FixedPoint inputs. "
+                            f"Receives {type(inputs)} inputs.")
+
+        # Quantize the weights
+        # Conv2DTranspose weight shape is (h, w, filters, channels) which cannot be quantized
+        # 'per-axis' as it is: the last dimension is not the output last dimension (i.e filters)
+        # so it must be transposed to (h, w, channels, filters) before quantization to have the
+        # appropriate frac_bits shape. Kernel values will then be transposed back to apply the
+        # Tensorflow operation.
+        kernel = self.weight_quantizer(tf.transpose(self.kernel, (0, 1, 3, 2), training))
+
+        # Promote inputs to avoid saturation, then align them
+        inputs, shift = inputs.promote(self.buffer_bitwidth).align()
+        self.input_shift(shift)
+
+        # Prepare deconvolution output shape
+        inputs_shape = tf.shape(inputs)
+        batch_size, height, width = inputs_shape[0], inputs_shape[1], inputs_shape[2]
+        kernel_h, kernel_w = self.kernel_size
+        stride_h, stride_w = self.strides
+
+        if self.output_padding is None:
+            out_pad_h = out_pad_w = None
+        else:
+            out_pad_h, out_pad_w = self.output_padding
+
+        # Infer the dynamic output shape
+        out_height = conv_utils.deconv_output_length(height,
+                                                     kernel_h,
+                                                     padding=self.padding,
+                                                     output_padding=out_pad_h,
+                                                     stride=stride_h,
+                                                     dilation=self.dilation_rate[0])
+        out_width = conv_utils.deconv_output_length(width,
+                                                    kernel_w,
+                                                    padding=self.padding,
+                                                    output_padding=out_pad_w,
+                                                    stride=stride_w,
+                                                    dilation=self.dilation_rate[1])
+
+        output_shape = (batch_size, out_height, out_width, self.filters)
+        output_shape_tensor = tf.stack(output_shape)
+
+        # Transpose back values so that they have the shape expected by the tensorflow op (h, w,
+        # filters, channels)
+        kernel_values = tf.transpose(kernel.values, (0, 1, 3, 2))
+
+        # Do convolution on values
+        conv_values = floor_through(
+            backend.conv2d_transpose(inputs.values, kernel_values, output_shape_tensor,
+                                     self.strides, self.padding, self.data_format,
+                                     self.dilation_rate))
+
+        if isinstance(kernel, QFloat):
+            # Multiply by the scale
+            conv_values *= kernel.scales.values
+
+        # Build a new FixedPoint
+        if isinstance(kernel, FixedPoint):
+            filters_frac_bits = kernel.frac_bits
+        else:
+            filters_frac_bits = kernel.scales.frac_bits
+        outputs = FixedPoint(conv_values, inputs.frac_bits + filters_frac_bits, inputs.value_bits)
+
+        if self.use_bias:
+            # Quantize biases
+            bias = self.bias_quantizer(self.bias, training)
+            # Align intermediate outputs and biases before adding them
+            outputs, shift = outputs.align(bias)
+            self.conv_shift(shift)
+            bias, shift = bias.promote(self.buffer_bitwidth).align(outputs)
+            self.bias_shift(shift)
+            outputs = tf.add(outputs, bias)
+
+        if self.out_quantizer is not None:
+            outputs = self.out_quantizer(outputs, training)
+        return outputs
+
+    def get_config(self):
+        config = super().get_config()
+        config["quant_config"] = self.quant_config
+        return config
