@@ -42,9 +42,9 @@ def deserialize_quant_object(quant_config, object_name, mandatory=True):
             object.
     """
     object_dict = quant_config.get(object_name)
-    if object_dict is None and mandatory:
-        raise KeyError(f"'{object_name}' should be specified.")
-    elif object_dict is None and not mandatory:
+    if object_dict is None:
+        if mandatory:
+            raise KeyError(f"'{object_name}' should be specified.")
         return None
 
     # Check that object_dict has the right keys
@@ -71,7 +71,7 @@ def deserialize_quant_object(quant_config, object_name, mandatory=True):
     deserialize_dict = object_dict.copy()
     deserialize_dict["name"] = object_name
     if "scale_bits" in deserialize_dict:
-        class_name = "Custom>UniformQuantizer"
+        class_name = "Custom>WeightQuantizer"
     else:
         class_name = "Custom>FixedPointQuantizer"
     deserialize_dict = {"class_name": class_name, "config": deserialize_dict}
