@@ -29,6 +29,7 @@ class QuantizedMaxPool2D(MaxPool2D):
     """A max pooling layer that operates on quantized inputs.
 
     """
+
     def call(self, inputs):
         # Raise an error if the inputs are not FixedPoint
         if not isinstance(inputs, FixedPoint):
@@ -44,30 +45,27 @@ class QuantizedGlobalAveragePooling2D(Calibrable, GlobalAveragePooling2D):
     """A global average pooling layer that operates on quantized inputs.
 
     """
+
     def __init__(self, quant_config={}, **kwargs):
         super().__init__(**kwargs)
-        self.spatial_size_inv = None
         self.quant_config = quant_config
         self.out_quantizer = deserialize_quant_object(self.quant_config, "output_quantizer", False)
 
     def build(self, input_shape):
         super().build(input_shape)
-        # Build the reciprocal of the spatial size
-        spatial_size_inv = 1 / (input_shape[1] * input_shape[2])
-        self.spatial_size_inv = FixedPoint.quantize(
-            spatial_size_inv, MAX_BUFFER_BITWIDTH, MAX_BUFFER_BITWIDTH)
+        # Build the spatial size
+        self.spatial_size = (input_shape[1] * input_shape[2])
 
     def call(self, inputs, training=None):
         # Raise an error if the inputs are not FixedPoint
         if not isinstance(inputs, FixedPoint):
-            raise TypeError(f"QuantizedMaxPool2D only accepts FixedPoint inputs. \
+            raise TypeError(f"QuantizedGlobalAveragePooling2D only accepts FixedPoint inputs. \
                              Receives {type(inputs)} inputs.")
 
-        # Promote input to prevent overflow in the reduce_sum op, then align
-        # them
+        # Promote input to prevent overflow in the reduce_sum op, then align them
         inputs, _ = inputs.promote(MAX_BUFFER_BITWIDTH).align()
         inputs_sum = tf.reduce_sum(inputs, axis=[1, 2], keepdims=self.keepdims)
-        inputs_mean = inputs_sum * self.spatial_size_inv
+        inputs_mean = inputs_sum / self.spatial_size
 
         if self.out_quantizer is not None:
             return self.out_quantizer(inputs_mean, training=training)
