@@ -51,13 +51,11 @@ class QuantizedSeparableConv2D(Calibrable, SeparableConv2D):
         self.out_quantizer = deserialize_quant_object(
             self.quant_config, "output_quantizer", False)
 
-        # Separable layer has two weights quantizers to handle different max values but they both
-        # use the same configuration
+        # Separable layer has two weights quantizers to handle different max values
         self.dw_weight_quantizer = deserialize_quant_object(
-            self.quant_config, "weight_quantizer", True)
-        # Duplicate configuration for pointwise, allows to create an object with a different name
-        pw_config = {"pw_weight_quantizer": self.quant_config["weight_quantizer"]}
-        self.pw_weight_quantizer = deserialize_quant_object(pw_config, "pw_weight_quantizer", True)
+            self.quant_config, "dw_weight_quantizer", True)
+        self.pw_weight_quantizer = deserialize_quant_object(
+            self.quant_config, "pw_weight_quantizer", True)
 
         if self.use_bias:
             self.bias_quantizer = deserialize_quant_object(
@@ -66,7 +64,7 @@ class QuantizedSeparableConv2D(Calibrable, SeparableConv2D):
         self.buffer_bitwidth = self.quant_config.get("buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
         assert self.buffer_bitwidth > 0, "The buffer_bitwidth must be a strictly positive integer."
         self.intermediate_quantizer = deserialize_quant_object(
-            self.quant_config, "intermediate_quantizer", True)
+            self.quant_config, "intermediate_quantizer", False)
 
     def call(self, inputs, training=None):
         # raise an error if the inputs are not FixedPoint or tf.Tensor
@@ -84,15 +82,18 @@ class QuantizedSeparableConv2D(Calibrable, SeparableConv2D):
             inputs = FixedPoint.quantize(inputs, 0, 8)
 
         inputs, _ = inputs.promote(self.buffer_bitwidth).align()
-        dw_outputs = backend.depthwise_conv2d(
+        dw_outputs_q = backend.depthwise_conv2d(
             inputs,
             depthwise_kernel,
             strides=self.strides,
             padding=self.padding,
             dilation_rate=self.dilation_rate,
             data_format=self.data_format)
-        dw_outputs_q = self.intermediate_quantizer(dw_outputs, training)
-        dw_outputs_q, _ = dw_outputs_q.promote(self.buffer_bitwidth).align()
+
+        if self.intermediate_quantizer is not None:
+            dw_outputs_q = self.intermediate_quantizer(dw_outputs_q, training)
+            dw_outputs_q, _ = dw_outputs_q.promote(self.buffer_bitwidth).align()
+
         outputs = tf.nn.convolution(
             dw_outputs_q,
             pointwise_kernel,
