@@ -27,7 +27,8 @@ from ..layers import (QuantizedDense, QuantizedLayerNormalization, QuantizedAddP
                       QuantizedReshape, QuantizedAttention, QuantizedConv2D, QuantizedMaxPool2D,
                       QuantizedGlobalAveragePooling2D, QuantizedExtractToken, Dequantizer,
                       Quantizer, QuantizedSeparableConv2D, QuantizedFlatten, QuantizedReciprocal,
-                      FixedPointQuantizer, UniformQuantizer, QuantizedConv2DTranspose)
+                      FixedPointQuantizer, WeightQuantizer, QuantizedConv2DTranspose,
+                      QuantizedShiftmax)
 
 # Mapper to match float layer with its quantized version
 CUSTOM_QLAYERS = {
@@ -39,10 +40,10 @@ CUSTOM_QLAYERS = {
     'GlobalAveragePooling2D': QuantizedGlobalAveragePooling2D,
     "ExtractToken": QuantizedExtractToken, "SeparableConv2D": QuantizedSeparableConv2D,
     "Flatten": QuantizedFlatten, "Reciprocal": QuantizedReciprocal, "PaddedConv2D": QuantizedConv2D,
-    "Conv2DTranspose": QuantizedConv2DTranspose
+    "Conv2DTranspose": QuantizedConv2DTranspose, "Shiftmax": QuantizedShiftmax
 }
 # List of Quantizer layer's that do not have a float layer representation
-NO_FLOAT_CUSTOM_QLAYERS = [Dequantizer, FixedPointQuantizer, UniformQuantizer]
+NO_FLOAT_CUSTOM_QLAYERS = [Dequantizer, FixedPointQuantizer, WeightQuantizer]
 
 
 def _handle_not_quantizable_layers(model):
