@@ -173,7 +173,8 @@ def apply_weights_to_model(model, weights, verbose=True):
             model. Defaults to True.
     """
     if len(weights) == 0:
-        raise ValueError("There is no weight to apply to the model.")
+        warnings.warn("There is no weight to apply to the model.")
+        return
 
     # Go through the dictionary of weights with each item
     for key, value in weights.items():
