@@ -16,7 +16,7 @@
 # ******************************************************************************
 import tensorflow as tf
 
-from .qtensor import QTensor, round_through, ceil_through
+from .qtensor import QTensor, round_through, ceil_through, pow2, ceil_log2
 from .fixed_point import FixedPoint
 
 
@@ -53,7 +53,7 @@ class QFloat(QTensor):
 
         x_int = round(x * int_max / float_max)
 
-        The resulting integer values are clipped to [-int_max, int_max].
+        The resulting integer values are clipped to [-int_max-1, int_max].
 
         Args:
             x (:obj:`tensorflow.Tensor`): a tensor of float values.
@@ -68,10 +68,10 @@ class QFloat(QTensor):
         scales = tf.cast(float_max, tf.float32) / QTensor.int_max(value_bits)
         # Make sure the scales are at least equal to the scales resolution,
         # otherwise they would be quantized to zero
-        scales_res = FixedPoint._pow2(-scales_bits)
+        scales_res = pow2(-scales_bits)
         scales = tf.math.maximum(scales, scales_res)
         # Quantize the scales to a FixedPoint
-        scales_int_bits = tf.math.maximum(0., ceil_through(tf.experimental.numpy.log2(scales)))
+        scales_int_bits = tf.math.maximum(0., ceil_log2(scales))
         scales_frac_bits = tf.clip_by_value(scales_bits - scales_int_bits,
                                             0, scales_bits)
         # We do not apply the default FixedPoint quantization algorithm, as it
@@ -79,7 +79,7 @@ class QFloat(QTensor):
         # are at least equal to the target scales. Should they be lower, this
         # would lead to a reduction of the QFloat quantization range and values
         # near float_max would be clipped.
-        scales_values = ceil_through(scales * FixedPoint._pow2(scales_frac_bits))
+        scales_values = ceil_through(scales * pow2(scales_frac_bits))
         fp_scales = FixedPoint(scales_values, scales_frac_bits, scales_bits)
         # Quantize the inputs using the approximated FixedPoint scales
         values = round_through(x / fp_scales.to_float())
