@@ -93,16 +93,15 @@ def fp_reduce_sum(input_tensor: FixedPoint, axis=None, keepdims=False, name=None
     Returns:
         :obj:`FixedPoint`: the summed FixedPoint.
     """
-    # The sum fractional bits is the max of all terms
-    s_frac_bits = tf.math.reduce_max(input_tensor.frac_bits)
-    # Align all terms on the same fractional bits
-    input_values = FixedPoint._lshift(
-        input_tensor.values, s_frac_bits - input_tensor.frac_bits)
+    # input_values must be quantized per-tensor
+    input_tensor.assert_per_tensor()
+
     # Reduce sum
     s_values = tf.math.reduce_sum(
-        input_values, axis, keepdims=keepdims, name=name)
+        input_tensor.values, axis, keepdims=keepdims, name=name)
+
     # Return a new FixedPoint
-    return FixedPoint(s_values, s_frac_bits, input_tensor.value_bits)
+    return FixedPoint(s_values, input_tensor.frac_bits, input_tensor.value_bits)
 
 
 @tf.experimental.dispatch_for_api(tf.linalg.matmul)
@@ -344,7 +343,7 @@ def fp_depthwise_conv2d(x: FixedPoint,
         x (obj:`FixedPoint`): input tensor.
         depthwise_kernel (:obj:`QTensor`): convolution kernel for the depthwise convolution.
         strides (tuple, optional): strides tuple (length 2). Defaults to (1, 1).
-        padding (str, optional): `"same"` or `"valid"`. Default to 'valid'.
+        padding (str, optional): `"same"` or `"valid"`. Defaults to 'valid'.
         data_format (str, optional): `"channels_last"` or `"channels_first"`. Defaults to None.
         dilation_rate (tuple, optional): tuple of integers, dilation rates for the separable
             convolution. Defaults to (1, 1).
@@ -353,12 +352,8 @@ def fp_depthwise_conv2d(x: FixedPoint,
         :obj:`FixedPoint`: output tensor.
 
     """
-    # Evaluate the maximum number of fractional bits
-    input_frac_bits = tf.reduce_max(x.frac_bits)
-
-    # If x has multiple fractional bits, we must align the values along the last dimension before
-    # the convolution collapses it
-    x = tf.cond(x.per_tensor, lambda: x, lambda: x << (input_frac_bits - x.frac_bits))
+    # x must be quantized per-tensor because the products are eventually summed
+    x.assert_per_tensor()
 
     if isinstance(depthwise_kernel, FixedPoint):
         filters_frac_bits = depthwise_kernel.frac_bits
@@ -374,7 +369,7 @@ def fp_depthwise_conv2d(x: FixedPoint,
         conv_values *= depthwise_kernel.scales.values
 
     # Return a new FixedPoint
-    return FixedPoint(conv_values, input_frac_bits + filters_frac_bits, x.value_bits)
+    return FixedPoint(conv_values, x.frac_bits + filters_frac_bits, x.value_bits)
 
 
 @tf.experimental.dispatch_for_api(tf.nn.convolution)
@@ -397,20 +392,14 @@ def fp_convolution(
             "NCW".  For N=2, the valid values are "NHWC" (default) and "NCHW".
             For N=3, the valid values are "NDHWC" (default) and "NCDHW".
         dilations (list, optional): Alias for dilation_rate. Sequence of N ints >= 1.
-            Specifies the filter upsampling/input downsampling rate. Default to None.
+            Specifies the filter upsampling/input downsampling rate. Defaults to None.
         name (str, optional): the name for the Tensorflow ops. Defaults to None.
 
     Returns:
         :obj:`FixedPoint`: a FixedPoint containing the output values.
     """
-    # Evaluate the maximum number of fractional bits
-    input_frac_bits = tf.reduce_max(input.frac_bits)
-
-    # If a has multiple fractional bits, we must align the values along the
-    # last dimension before the convolution collapses it
-    input = tf.cond(input.per_tensor,
-                    lambda: input,
-                    lambda: input << (input_frac_bits - input.frac_bits))
+    # Input must be quantized per-tensor because the products are eventually summed
+    input.assert_per_tensor()
 
     if isinstance(filters, FixedPoint):
         filters_frac_bits = filters.frac_bits
@@ -426,4 +415,4 @@ def fp_convolution(
         conv_values *= filters.scales.values
 
     # Return a new FixedPoint
-    return FixedPoint(conv_values, input_frac_bits + filters_frac_bits, input.value_bits)
+    return FixedPoint(conv_values, input.frac_bits + filters_frac_bits, input.value_bits)
